//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <math.h>
#include <stdio.h>
#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TOpenGLAPPanel"
#pragma resource "*.dfm"
TForm1 *Form1;

typedef struct tagVERTEX {
	float x, y, z;
	float u, v;
    } VERTEX;

typedef struct tagTRIANGLE {
	VERTEX vertex[3];
    } TRIANGLE;

typedef struct tagSECTOR {
	int numtriangles;
	TRIANGLE* triangle;
    } SECTOR;

bool	blend;				// Blending ON/OFF

const float piover180 = 0.0174532925f;
float heading;
float xpos;
float zpos;

GLfloat	yrot;				// Y Rotation
GLfloat walkbias = 0;
GLfloat walkbiasangle = 0;
GLfloat lookupdown = 0.0f;
GLfloat	z=0.0f;				// Depth Into The Screen

GLuint	filter = 0;	    	// Which Filter To Use
GLuint	texture[3];			// Storage For 3 Textures


SECTOR sector1;				// Our Model Goes Here:

//---------------------------------------------------------------------------
void readstr(FILE *f,char *string);
void SetupWorld();


//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void readstr(FILE *f,char *string)
{
	do {
        fgets(string, 255, f);
	} while ((string[0] == '/') || (string[0] == '\n'));
}

//---------------------------------------------------------------------------
void SetupWorld()
{
	float x, y, z, u, v;
	int numtriangles;
	FILE *filein;
	char oneline[255];
	filein = fopen("data/world.txt", "rt");				// File To Load World Data From

	readstr(filein,oneline);
	sscanf(oneline, "NUMPOLLIES %d\n", &numtriangles);

	sector1.triangle = new TRIANGLE[numtriangles];
	sector1.numtriangles = numtriangles;
	for (int loop = 0; loop < numtriangles; loop++)	{
		for (int vert = 0; vert < 3; vert++) {
			readstr(filein,oneline);
			sscanf(oneline, "%f %f %f %f %f %f %f", &x, &y, &z, &u, &v);
			sector1.triangle[loop].vertex[vert].x = x;
			sector1.triangle[loop].vertex[vert].y = y;
			sector1.triangle[loop].vertex[vert].z = z;
			sector1.triangle[loop].vertex[vert].u = u;
			sector1.triangle[loop].vertex[vert].v = v;
		    }
	    }
	fclose(filein);
}

//---------------------------------------------------------------------------
int __fastcall  TForm1::LoadGLTextures() // Load Bitmaps And Convert To Textures
{
	int Status=FALSE;									// Status Indicator
    BMPTexture  *BMPTex = NULL;

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
    BMPTex = OpenGLAPPanel1->LoadBMPTexture ("Data/mud.bmp", false);
    if (BMPTex)	{
        Status=TRUE;                            // Set The Status To TRUE

        glGenTextures(3, &texture[0]);          // Create Three Textures

        // Create Nearest Filtered Texture
        glBindTexture(GL_TEXTURE_2D, texture[0]);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
        glTexImage2D(GL_TEXTURE_2D, 0, 3, BMPTex->GetWidth(), BMPTex->GetHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE, BMPTex->GetPixels());

        // Create Linear Filtered Texture
        glBindTexture(GL_TEXTURE_2D, texture[1]);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D, 0, 3, BMPTex->GetWidth(), BMPTex->GetHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE, BMPTex->GetPixels());

        // Create MipMapped Texture
        glBindTexture(GL_TEXTURE_2D, texture[2]);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, BMPTex->GetWidth(), BMPTex->GetHeight(), GL_RGB, GL_UNSIGNED_BYTE, BMPTex->GetPixels());
        delete BMPTex;
    	}
	return Status;										// Return The Status
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLAPPanel1Init(TObject *Sender)
{
	if (!LoadGLTextures()) {							// Jump To Texture Loading Routine
		return;									        // If Texture Didn't Load Return FALSE
	    }


	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Set The Blending Function For Translucency
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);				// This Will Clear The Background Color To Black
	glClearDepth(1.0);									// Enables Clearing Of The Depth Buffer
	glDepthFunc(GL_LESS);								// The Type Of Depth Test To Do
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glShadeModel(GL_SMOOTH);							// Enables Smooth Color Shading
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

	SetupWorld();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLAPPanel1Paint(TObject *Sender)
{
	GLfloat x_m, y_m, z_m, u_m, v_m;
	GLfloat xtrans = -xpos;
	GLfloat ztrans = -zpos;
	GLfloat ytrans = -walkbias-0.25f;
	GLfloat sceneroty = 360.0f - yrot;

	int numtriangles;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();									// Reset The View

	glRotatef(lookupdown,1.0f,0,0);
	glRotatef(sceneroty,0,1.0f,0);
	
	glTranslatef(xtrans, ytrans, ztrans);
	glBindTexture(GL_TEXTURE_2D, texture[filter]);
	
	numtriangles = sector1.numtriangles;
	
	// Process Each Triangle
	for (int loop_m = 0; loop_m < numtriangles; loop_m++)	{
		glBegin(GL_TRIANGLES);
			glNormal3f( 0.0f, 0.0f, 1.0f);
			x_m = sector1.triangle[loop_m].vertex[0].x;
			y_m = sector1.triangle[loop_m].vertex[0].y;
			z_m = sector1.triangle[loop_m].vertex[0].z;
			u_m = sector1.triangle[loop_m].vertex[0].u;
			v_m = sector1.triangle[loop_m].vertex[0].v;
			glTexCoord2f(u_m,v_m); glVertex3f(x_m,y_m,z_m);

			x_m = sector1.triangle[loop_m].vertex[1].x;
			y_m = sector1.triangle[loop_m].vertex[1].y;
			z_m = sector1.triangle[loop_m].vertex[1].z;
			u_m = sector1.triangle[loop_m].vertex[1].u;
			v_m = sector1.triangle[loop_m].vertex[1].v;
			glTexCoord2f(u_m,v_m); glVertex3f(x_m,y_m,z_m);

			x_m = sector1.triangle[loop_m].vertex[2].x;
			y_m = sector1.triangle[loop_m].vertex[2].y;
			z_m = sector1.triangle[loop_m].vertex[2].z;
			u_m = sector1.triangle[loop_m].vertex[2].u;
			v_m = sector1.triangle[loop_m].vertex[2].v;
			glTexCoord2f(u_m,v_m); glVertex3f(x_m,y_m,z_m);
		glEnd();
	    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLAPPanel1Resize(TObject *Sender)
{
	if (OpenGLAPPanel1->Height==0)										// Prevent A Divide By Zero By
	{
		OpenGLAPPanel1->Height=1;										// Making Height Equal One
	}

	glViewport(0,0,OpenGLAPPanel1->Width,OpenGLAPPanel1->Height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)OpenGLAPPanel1->Width/(GLfloat)OpenGLAPPanel1->Height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    switch (Key) {
        case 'B':
            blend=!blend;
            if (blend) {
                glEnable(GL_BLEND);
                glDisable(GL_DEPTH_TEST);
                }
            else {
                glDisable(GL_BLEND);
                glEnable(GL_DEPTH_TEST);
                }
            break;
        case 'F':
            filter+=1;
            if (filter>2) {
                filter = 0;
                }
            break;
        case VK_PRIOR:
            z-=0.02f;
            lookupdown-= 1.0f;
            break;
        case VK_NEXT:
            z+=0.02f;;
            lookupdown+= 1.0f;
            break;
        case VK_UP:
            xpos -= (float)sin(heading*piover180) * 0.05f;
            zpos -= (float)cos(heading*piover180) * 0.05f;
            if (walkbiasangle >= 359.0f) {
                walkbiasangle = 0.0f;
                }
            else {
                walkbiasangle+= 10;
                }
            walkbias = (float)sin(walkbiasangle * piover180)/20.0f;
            break;
        case VK_DOWN:
            xpos += (float)sin(heading*piover180) * 0.05f;
            zpos += (float)cos(heading*piover180) * 0.05f;
            if (walkbiasangle <= 1.0f) {
                walkbiasangle = 359.0f;
                }
            else {
                walkbiasangle-= 10;
                }
            walkbias = (float)sin(walkbiasangle * piover180)/20.0f;
            break;
        case VK_LEFT:
            heading += 1.0f;
            yrot = heading;
            break;
        case VK_RIGHT:
            heading -= 1.0f;
            yrot = heading;
            break;
        }
    OpenGLAPPanel1->Repaint ();
}
//---------------------------------------------------------------------------

